;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(messsage "Loading xwem custom load")

(custom-add-loads 'xwem '("xwem-main" "xwem-clients" "xwem-faces" "xwem-frame" "xwem-keyboard" "xwem-launcher" "xwem-minibuffer" "xwem-misc" "xwem-special" "xwem-strokes" "xwem-tabbing" "xwem-tray" "xwem-win"))
(custom-add-loads 'xwem-cursor '("xwem-misc"))
(custom-add-loads 'xwem-special '("xwem-special"))
(custom-add-loads 'applications '("xwem-main"))
(custom-add-loads 'xwem-minibuffer '("xwem-minibuffer"))
(custom-add-loads 'xwem-faces '("xwem-faces" "xwem-frame" "xwem-strokes" "xwem-tabbing"))
(custom-add-loads 'xwem-launcher '("xwem-launcher"))
(custom-add-loads 'xwem-keys '("xwem-keyboard"))
(custom-add-loads 'xwem-cl '("xwem-clients" "xwem-manage"))
(custom-add-loads 'xwem-frame '("xwem-frame"))
(custom-add-loads 'xwem-keyboard '("xwem-keyboard" "xwem-keymacro"))
(custom-add-loads 'xwem-win '("xwem-win"))
(custom-add-loads 'xwem-tray '("xwem-tray"))
(custom-add-loads 'xwem-strokes '("xwem-strokes"))
(custom-add-loads 'xwem-tab '("xwem-tabbing"))
(custom-add-loads 'xwem-storkes '("xwem-strokes"))
(custom-add-loads 'xwem-misc '("xwem-misc"))

;;; custom-load.el ends here
